package demo.sell.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.core.model.XmlCallbackResponse;
import com.huifu.jh.sell.model.GcbFxSettleDetailApplyRequest;
import com.huifu.jh.sell.model.GcbFxSettleDetailApplyRequestDetail;
import com.huifu.jh.sell.model.GcbFxSettleDetailApplyResponse;
import com.huifu.jh.sell.model.GcbFxSettleDetailReviewResult;
import com.huifu.jh.sell.model.GcbFxSettleDetailReviewResultResponse;
import com.huifu.jh.sell.model.GcbFxT0SettleResult;
import com.huifu.jh.sell.model.GcbFxT0SettleResultResponse;

import demo.sell.config.AppConfig;

/**
 * @author fang.wei
 * @Description 结汇明细备案DEMO
 * @createTime 2021年10月30日 23:47:00
 */
@Controller
public class GcbFxSettleDetailApplyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    @GetMapping("/showGcbFxSettleDetailApply.do")
    public String showGcbFxSettleDetailApply(HttpServletRequest request, HttpServletResponse response, Model model) {
        GcbFxSettleDetailApplyRequest req = new GcbFxSettleDetailApplyRequest();
        req.setMerchantId("1001215986501");
        req.setTerminalId("JHDF001");
        req.setBatchNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        req.setOrdNum("1/1");
        req.setRequestTime(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
        req.setPayCur("USD");
        req.setPayTotalCnt("1");
        req.setPayTotalAmt("1000");
        req.setSumCnt("1");
        req.setSumAmt("1000");
        req.setReviewUrl("http://localhost:8080/receiveSettleDetailApplyResult.do");
        req.setResultUrl("http://localhost:8080/receiveSettleDetailApplyResult.do");
        req.setDetails(mockDetails());
        model.addAttribute("req", req);
        model.addAttribute("jsonDetails", JSON.toJSONString(req.getDetails()));
        return "GcbFxSettleDetailApply";
    }

    private List<GcbFxSettleDetailApplyRequestDetail> mockDetails() {
        List<GcbFxSettleDetailApplyRequestDetail> list = new ArrayList<>();
        GcbFxSettleDetailApplyRequestDetail d = new GcbFxSettleDetailApplyRequestDetail();
        d.setSeqId(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        d.setOrderId(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        d.setOrderTime(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
        d.setPayeeType("P");
        d.setPayeeIdtype("00");
        d.setPayeeIdno("320125198805233308");
        d.setPayeeName("李财水");
        d.setPayeeAcct("6465767887999");
        d.setBankId("CMB");
        d.setPayCur("USD");
        d.setPayAmt("1000");
        d.setProductDesc("{\"customInfo\":{\"addr\":\"客户地址\",\"name\":\"客户名称\",\"tel\":\"客户联系电话\"},\"goodInfo\":[{\"cnt\":\"3\",\"name\":\"货物名称 1\",\"price\":\"12.31\"},{\"cnt\":\"1\",\"name\":\"货物名称2\",\"price\":\"100.31\"}],\"storeInfo\":{\"storeId\":\"amazon_eq3142412342\",\"storelink\": \"12010\"},\"totalCnt\":\"4\",\"cur\":\"USD\",\"totalAmt\":\"900\",\"logInfo\":{\"logCompany\":\"物流公司名称\",\"logNo\":\"物流单号\"},\"productId\":\"100200\"}");
        d.setRemark("remark");
        list.add(d);
        return list;
    }

    @PostMapping("/doGcbFxSettleDetailApply.do")
    @ResponseBody
    public String doGcbFxSettleDetailApply(HttpServletRequest request, HttpServletResponse response, Model model, GcbFxSettleDetailApplyRequest req) throws Exception {

        // 服务url(测试环境)
        String serverUrl = "https://hfgj.testpnr.com/sell/receivesell.htm";
        // 服务url(生产环境)
        //	String serverUrl = "https://global.chinapnr.com/sell/receivesell.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        String details = request.getParameter("jsonDetails");
        req.setDetails(JSON.parseArray(details, GcbFxSettleDetailApplyRequestDetail.class));

        // 发送请求
        GcbFxSettleDetailApplyResponse resp = gcbPayClient.execute(req);
        logger.info("resp => " + new String(resp.getResponseBody()));

        // TODO 业务处理
        // ......

        String respTxt = "响应信息<br />" + resp.toString();
        return respTxt;
    }

    @PostMapping("/receiveSettleDetailApplyResult.do")
    public String receiveSettleDetailApplyResult(HttpServletRequest request, HttpServletResponse response, Model model) {
        // 实例化通知解析器
        GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

        try {
            GcbFxSettleDetailReviewResult result = gcbMessageParser.parse(request, GcbFxSettleDetailReviewResult.class);
            logger.info("result -> " + result.toString());

            // 向服务端返回接收成功
            XmlCallbackResponse resp = buildResponse(result);
            response.getWriter().write(resp.toXml());
        } catch (Exception e) {
            e.printStackTrace();
            // TODO
        }

        return null;
    }

    private static XmlCallbackResponse buildResponse(GcbFxSettleDetailReviewResult result) {
        GcbFxSettleDetailReviewResultResponse resp = new GcbFxSettleDetailReviewResultResponse();
        resp.setMerchantId(result.getMerchantId());
        resp.setTerminalId(result.getTerminalId());
        resp.setProductType(result.getProductType());
        resp.setVersion(result.getVersion());
        resp.setTrxType(result.getTrxType());
        resp.setResult("000000");
        return resp;
    }

    @PostMapping("/receiveT0SettleResult.do")
    public String receiveT0SettleResult(HttpServletRequest request, HttpServletResponse response, Model model) {
        // 实例化通知解析器
        GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

        try {
            GcbFxT0SettleResult result = gcbMessageParser.parse(request, GcbFxT0SettleResult.class);
            logger.info("result -> " + result.toString());

            // 向服务端返回接收成功
            XmlCallbackResponse resp = buildT0SettleResponse(result);
            response.getWriter().write(resp.toXml());
        } catch (Exception e) {
            e.printStackTrace();
            // TODO
        }

        return null;
    }

    private XmlCallbackResponse buildT0SettleResponse(GcbFxT0SettleResult result) {
        GcbFxT0SettleResultResponse resp = new GcbFxT0SettleResultResponse();
        resp.setMerchantId(result.getMerchantId());
        resp.setTerminalId(result.getTerminalId());
        resp.setProductType(result.getProductType());
        resp.setVersion(result.getVersion());
        resp.setTrxType(result.getTrxType());
        resp.setResult("000000");
        return resp;
    }

}